# -*- coding=utf-8 -*-

import six
import logging
import os
import traceback
import json
import finbull.error
from urllib import parse

LOG_DIR = "./logs/"
MODULE = "app"
FOTMAT = "%(levelname)s: %(asctime)s %(f_module)s [filename=%(f_filename)s lineno=%(f_lineno)d " \
         "process=%(process)d thread=%(thread)d thread_name=%(threadName)s created=%(created)f" \
         " msecs=%(msecs)d %(message)s]"

ACCESS_FORMAT = "%(asctime)s %(module)s %(process)d [%(message)s]"
LEVEL_NOTICE = 21
LEVEL_FATAL = 41
logging.addLevelName(LEVEL_NOTICE, "NOTICE")
logging.addLevelName(LEVEL_FATAL, "FATAL")

"""
app_log is suitable for human reading.

app_omg_log is suitable for statistics and monitor,
each request ONLY has one line log.

"""
app_log = logging.getLogger("finbull.log.app")
app_log_wf = logging.getLogger("finbull.log.app_wf")
app_service_log = logging.getLogger("finbull.log.app.service")
app_service_log_wf = logging.getLogger("finbull.log.app.service.wf")


def _get_current_trackback():
    """
    get current traceback

    traceback is wrong in the log file.
    so we should find it by myself.
    """
    traces = traceback.extract_stack()
    for i, trace in enumerate(traces):
        if trace[0].endswith("finbull/log.py") or\
                trace[0].endswith("finbull/log.pyc") or\
                trace[0].endswith("finbull/log.pyo"):
            if trace[2] in [
                "debug", "notice", "warning", "fatal",
                "service_debug", "service_notice",
                    "service_warning", "service_fatal"]:
                return traces[i - 1]
    # can not find any current trace.
    # return the default
    return traces[0]


def _json_parser(v):
    if isinstance(v, (list, dict)):
        # print json.dumps(v, default=repr)
        return json.dumps(v, default=repr)
    return str(v)


_default_parse = repr


def set_parser(parser):
    """ set parser
    """
    global _default_parse
    _default_parse = parser


"""
包装后的logger 会默认打印当前的trace
所有的日志 都会是log的行号 和 stack 信息
所以这里都统一替换一下

Args:
    parser (TYPE): Description
"""


class FilenameFilter(logging.Filter):
    """Summary
    FilenameFilter
    """

    def filter(self, record):
        """Summary
        filter
        """
        record.f_filename = os.path.abspath(_get_current_trackback()[0])
        return True


class ModuleFilter(logging.Filter):
    """Summary
    ModuleFilter
    """

    def filter(self, record):
        """Summary
        filter
        """
        record.f_module = _get_current_trackback()[2]
        return True


class LinenoFilter(logging.Filter):
    """Summary
    LinenoFilter
    """

    def filter(self, record):
        """Summary
        filter
        """
        record.f_lineno = _get_current_trackback()[1]
        return True


def init(log_dir=LOG_DIR, module=MODULE, debug=True):
    """Summary
    feed log init.

    Attribute:
        log_dir: log dir
        module: log name
        debug:
    """
    try:
        if not os.path.isdir(log_dir):
            os.makedirs(log_dir)
    except:
        pass

    if debug is True:
        app_log.setLevel(logging.DEBUG)
        app_log_wf.setLevel(logging.DEBUG)
        app_service_log.setLevel(logging.DEBUG)
        app_service_log_wf.setLevel(logging.DEBUG)
    else:
        app_log.setLevel(logging.INFO)
        app_log_wf.setLevel(logging.WARNING)
        app_service_log.setLevel(logging.INFO)
        app_service_log_wf.setLevel(logging.WARNING)

    formatter = logging.Formatter(FOTMAT)
    filename_filter = FilenameFilter()
    module_filter = ModuleFilter()
    lineno_filter = LinenoFilter()

    # app log
    app_log_hd = logging.handlers.TimedRotatingFileHandler(
        log_dir + "/" + module + ".log", "H", 1, 24 * 7)
    app_log_hd.suffix = "%Y%m%d%H"
    app_log_hd.setFormatter(formatter)
    app_log.addHandler(app_log_hd)
    app_log.addFilter(filename_filter)
    app_log.addFilter(module_filter)
    app_log.addFilter(lineno_filter)
    app_log.propagate = False

    # app warning fatal log
    app_log_wf_hd = logging.handlers.TimedRotatingFileHandler(
        log_dir + "/" + module + ".log.wf", "H", 1, 24 * 7)
    app_log_wf_hd.suffix = "%Y%m%d%H"
    app_log_wf_hd.setFormatter(formatter)
    app_log_wf.addHandler(app_log_wf_hd)
    app_log_wf.addFilter(filename_filter)
    app_log_wf.addFilter(module_filter)
    app_log_wf.addFilter(lineno_filter)
    app_log_wf.propagate = False

    # app service log
    app_service_log_hd = logging.handlers.TimedRotatingFileHandler(
        log_dir + "/" + module + ".service.log", "H", 1, 24 * 7)
    app_service_log_hd.suffix = "%Y%m%d%H"
    app_service_log_hd.setFormatter(formatter)
    app_service_log.addHandler(app_service_log_hd)
    app_service_log.addFilter(filename_filter)
    app_service_log.addFilter(module_filter)
    app_service_log.addFilter(lineno_filter)
    app_service_log.propagate = False

    # app service warning fatal log
    app_service_log_wf_hd = logging.handlers.TimedRotatingFileHandler(
        log_dir + "/" + module + ".service.log.wf", "H", 1, 24 * 7)
    app_service_log_wf_hd.suffix = "%Y%m%d%H"
    app_service_log_wf_hd.setFormatter(formatter)
    app_service_log_wf.addHandler(app_service_log_wf_hd)
    app_service_log_wf.addFilter(filename_filter)
    app_service_log_wf.addFilter(module_filter)
    app_service_log_wf.addFilter(lineno_filter)
    app_service_log_wf.propagate = False


def _urlencode(value):
    """Summary
    _urlencode
    """
    return parse.quote_plus(value)


def debug(*args):
    """Summary
    add a DEBUG log
    """
    if isinstance(args[0], six.string_types):
        value = "None" if len(args) <= 1 else args[1]
        app_log.debug("%s=%s" % (args[0], value))

    elif isinstance(args[0], dict):
        app_log.debug(' '.join(['%s=%s' % (key, value)
                                for (key, value) in args[0].items()]))

    else:
        raise finbull.error.BaseError(
            errno=finbull.error.ERRNO_FRAMEWORK,
            errmsg="can not call log.debug, parameters error. need str or dict."
        )


def service_debug(*args):
    """Summary
    add a service DEBUG log
    """
    if isinstance(args[0], six.string_types):
        value = "None" if len(args) <= 1 else args[1]
        app_service_log.debug("%s=%s" % (args[0], _urlencode(str(value))))

    elif isinstance(args[0], dict):
        app_service_log.debug(' '.join(['%s=%s' % (key, _urlencode(str(value)))
                                        for (key, value) in args[0].items()]))

    else:
        raise finbull.error.BaseError(
            errno=finbull.error.ERRNO_FRAMEWORK,
            errmsg="can not call log.service_debug, parameters error. need str or dict."
        )


def notice(*args, **kwargs):
    """Summary
    add a NOTICE log
    """
    # try to get a parse function
    _parse = kwargs.get("parse_function", _default_parse)
    key, value = "msg", "None"
    if len(args) <= 0:
        key, value = "msg", "None"
    # if the first args is a dict, parse it
    elif len(args) <= 1:
        if isinstance(args[0], dict):
            app_log.log(LEVEL_NOTICE,
                        ' '.join(['%s=%s' % (k, _parse(v))
                                  for (k, v) in args[0].items()]))
            return
        else:
            key, value = "msg", args[0]
    else:
        key, value = args[0], args[1]

    if isinstance(value, six.string_types):
        app_log.log(LEVEL_NOTICE, "%s=%s" % (key, value))
    elif isinstance(value, (list, dict)):
        app_log.log(LEVEL_NOTICE, "%s=%s" % (key, _parse(value)))
    else:
        raise finbull.error.BaseError(
            errno=finbull.error.ERRNO_FRAMEWORK,
            errmsg="can not call log.notice, parameters error. need str or dict."
        )


def service_notice(*args):
    """Summary
    add a service NOTICE log
    """
    if isinstance(args[0], six.string_types):
        value = "None" if len(args) <= 1 else args[1]
        app_service_log.log(LEVEL_NOTICE, "%s=%s" % (args[0], _urlencode(str(value))))

    elif isinstance(args[0], dict):
        app_service_log.log(LEVEL_NOTICE,
                            ' '.join(['%s=%s' % (key, _urlencode(str(value)))
                                      for (key, value) in args[0].items()]))

    else:
        raise finbull.error.BaseError(
            errno=finbull.error.ERRNO_FRAMEWORK,
            errmsg="can not call log.notice, parameters error. need str or dict."
        )


def warning(*args):
    """Summary
    add a WARNING log
    """
    if isinstance(args[0], six.string_types):
        value = "None" if len(args) <= 1 else args[1]
        app_log_wf.warning("%s=%s" % (args[0], value))

    elif isinstance(args[0], dict):
        app_log_wf.warning(' '.join(['%s=%s' % (key, value)
                                     for (key, value) in args[0].items()]))

    else:
        raise finbull.error.BaseError(
            errno=finbull.error.ERRNO_FRAMEWORK,
            errmsg="can not call log.warning, parameters error. need str or dict."
        )


def service_warning(*args):
    """Summary
    add a service WARNING log
    """
    if isinstance(args[0], six.string_types):
        value = "None" if len(args) <= 1 else args[1]
        app_service_log_wf.warning("%s=%s" % (args[0], _urlencode(str(value))))

    elif isinstance(args[0], dict):
        app_service_log_wf.warning(' '.join(['%s=%s' % (key, _urlencode(str(value)))
                                             for (key, value) in args[0].items()]))

    else:
        raise finbull.error.BaseError(
            errno=finbull.error.ERRNO_FRAMEWORK,
            errmsg="can not call log.warning, parameters error. need str or dict."
        )


def fatal(*args):
    """Summary
    add a FATAL log
    """
    if isinstance(args[0], six.string_types):
        value = "None" if len(args) <= 1 else args[1]
        app_log_wf.log(LEVEL_FATAL, "%s=%s" % (args[0], value))

    elif isinstance(args[0], dict):
        app_log_wf.log(LEVEL_FATAL, ' '.join(['%s=%s' % (key, value)
                                              for (key, value) in args[0].items()]))

    else:
        raise finbull.error.BaseError(
            errno=finbull.error.ERRNO_FRAMEWORK,
            errmsg="can not call log.fatal, parameters error. need str or dict."
        )


def service_fatal(*args):
    """Summary
    add a service FATAL log
    """
    if isinstance(args[0], six.string_types):
        value = "None" if len(args) <= 1 else args[1]
        app_service_log_wf.log(LEVEL_FATAL, "%s=%s" % (args[0], _urlencode(str(value))))

    elif isinstance(args[0], dict):
        app_service_log_wf.log(LEVEL_FATAL,
                               ' '.join(['%s=%s' % (key, _urlencode(str(value)))
                                         for (key, value) in args[0].items()]))

    else:
        raise finbull.error.BaseError(
            errno=finbull.error.ERRNO_FRAMEWORK,
            errmsg="can not call log.fatal, parameters error. need str or dict."
        )


if __name__ == '__main__':
    import tornado.web
    import tornado.ioloop
    from tornado.httpclient import AsyncHTTPClient

    init()

    class MainHandler(tornado.web.RequestHandler):
        """Summary
        MainHandler
        """

        def get(self):
            """Summary

            Returns:
                TYPE: Description
            """
            notice("port", "start 8999")
            notice("aaa", "bbfdl;akr3i2u53214%#&^%&^%#$@Wgfdlk\
                sjf红额外哦IQ缴费机啊街坊大姐唉算了；就发")

            warning("get_error_func", "fdaljfda辅导课辣椒水放假的撒娇 ")
            fatal("get_user_info", "failed to get user infomation")

            # test unicode
            notice("fdafda", u"地方大快速链接++++")

            # test dict
            notice({"a": "b", "c": "d"})

            self.write("Hello, world")

    app = tornado.web.Application([
        (r"/", MainHandler),
    ])

    app.listen(8999)

    tornado.ioloop.IOLoop.current().run_sync(
        lambda: AsyncHTTPClient().fetch(
            'http://localhost:8999/', follow_redirects=False, raise_error=False))

    tornado.ioloop.IOLoop.current().run_sync(
        lambda: AsyncHTTPClient().fetch(
            'http://localhost:8999/', follow_redirects=False, raise_error=False))



# -*- coding: utf-8 -*-
# Copyright (c) 2019 hcyjs.com, Inc. All Rights Reserved

import random
import time

import tornadis
import tornado.gen
import tornado.ioloop
import finbull.error
import finbull.log


class RedisDB(object):
    """
    RedisDB
    """
    # client number in connect pool
    MAX_CLIENT_CLIENT = 20
    RETRY_TIMES = 3

    def __init__(self, ip_list=None, consts=None, **client_kwargs):
        """
        init and connect at first time
        """
        self.client_kwargs = client_kwargs
        self.consts = consts or {}
        self.ip_list = ip_list

        if self.ip_list is None:
            raise finbull.error.BaseError(
                errno=finbull.error.ERRNO_FRAMEWORK,
                errmsg="finbull is empty both."
            )

        if not isinstance(self.ip_list, list):
            self.ip_list = [self.ip_list]
        self.pool = None

        # connect at first time
        tornado.ioloop.IOLoop.current().run_sync(self.restart_pool)

    @tornado.gen.coroutine
    def restart_pool(self):
        """start a connect pool

        Returns:
            TYPE: Description
        """
        if self.pool is not None:
            self.pool.destory()
        address = random.choice(self.ip_list)
        (ip, port) = address.split(":")
        self.pool = tornadis.ClientPool(
            max_size=self.MAX_CLIENT_CLIENT,
            host=ip,
            port=int(port),
            **self.client_kwargs)

    @tornado.gen.coroutine
    def _get_connected_client(self):
        """
        get a connected client from the pool
        if I can not find any client in the pool
        raise the `finbull.error.BaseError`
        """
        for i in range(0, self.RETRY_TIMES):
            client = yield self.pool.get_connected_client()
            if isinstance(client, tornadis.exceptions.ClientError):
                yield self.restart_pool()
            else:
                raise tornado.gen.Return(client)

        raise finbull.error.BaseError(
            errno=finbull.error.ERRNO_FRAMEWORK,
            errmsg="no redis can be connected."
        )

    @tornado.gen.coroutine
    def get(self, key):
        """Summary

        Args:
            key (TYPE): Description

        Returns:
            TYPE: Description
        """
        _start = int(round(time.time() * 1000))
        client = yield self._get_connected_client()
        result = yield client.call("GET", key)
        self.pool.release_client(client)
        _stop = int(round(time.time() * 1000)) - _start

        finbull.log.service_debug({
            "redis_log": "redis requested",
            "method": "GET",
            "key": key,
            "result": result,
            "time": _stop,
        })
        finbull.log.service_notice({
            "redis_log": "redis requested",
            "method": "GET",
            "key": key,
            "time": _stop,
        })
        raise tornado.gen.Return(result)

    @tornado.gen.coroutine
    def set(self, key, value, cache_time=None):
        """Summary

        Args:
            key (TYPE): Description

        Returns:
            TYPE: Description
        """
        _start = int(round(time.time() * 1000))
        client = yield self._get_connected_client()
        if cache_time is not None:
            result = yield client.call("SET", key, value, "EX", cache_time)
        else:
            result = yield client.call("SET", key, value)
        self.pool.release_client(client)
        _stop = int(round(time.time() * 1000)) - _start

        finbull.log.service_debug({
            "redis_log": "redis requested",
            "method": "SET",
            "key": key,
            "value": value,
            "result": result,
            "time": _stop,
        })
        finbull.log.service_notice({
            "redis_log": "redis requested",
            "method": "SET",
            "key": key,
            "time": _stop,
        })
        raise tornado.gen.Return(result)

    @tornado.gen.coroutine
    def zadd(self, key, valuelist):
        """add

        Args:
            key (TYPE): Description
            valuelist (TYPE): Description

        Returns:
            TYPE: Description

        Raises:
            gen.Return: Description
        """
        if isinstance(valuelist, list):
            valuelist = map(str, valuelist)

        _start = int(round(time.time() * 1000))
        client = yield self._get_connected_client()
        result = yield client.call("ZADD", key, *valuelist)
        self.pool.release_client(client)
        _stop = int(round(time.time() * 1000)) - _start

        finbull.log.service_debug({
            "redis_log": "redis requested",
            "method": "ZADD",
            "key": key,
            "value": valuelist,
            "result": result,
            "time": _stop,
        })
        finbull.log.service_notice({
            "redis_log": "redis requested",
            "method": "ZADD",
            "key": key,
            "time": _stop,
        })
        raise tornado.gen.Return(result)

    @tornado.gen.coroutine
    def zrevrange(self, key, count, withscores=False):
        """range

        Args:
            key (TYPE): Description
            valuelist (TYPE): Description

        Returns:
            TYPE: Description

        Raises:
            gen.Return: Description
        """
        _start = int(round(time.time() * 1000))
        client = yield self._get_connected_client()
        result = yield client.call("ZREVRANGE", key, 0, count - 1)
        self.pool.release_client(client)
        _stop = int(round(time.time() * 1000)) - _start

        finbull.log.service_debug({
            "redis_log": "redis requested",
            "method": "ZREVRANGE",
            "key": key,
            "result": result,
            "time": _stop,
        })
        finbull.log.service_notice({
            "redis_log": "redis requested",
            "method": "ZREVRANGE",
            "key": key,
            "time": _stop,
        })
        raise tornado.gen.Return(result)

    @tornado.gen.coroutine
    def sadd(self, key, valuelist):
        """add

        Args:
            key (TYPE): Description
            valuelist (TYPE): Description

        Returns:
            TYPE: Description

        Raises:
            gen.Return: Description
        """
        if isinstance(valuelist, list):
            valuelist = map(str, valuelist)

        _start = int(round(time.time() * 1000))
        client = yield self._get_connected_client()
        result = yield client.call("SADD", key, *valuelist)
        self.pool.release_client(client)
        _stop = int(round(time.time() * 1000)) - _start

        finbull.log.service_debug({
            "redis_log": "redis requested",
            "method": "SADD",
            "key": key,
            "value": valuelist,
            "result": result,
            "time": _stop,
        })
        finbull.log.service_notice({
            "redis_log": "redis requested",
            "method": "SADD",
            "key": key,
            "time": _stop,
        })
        raise tornado.gen.Return(result)

    @tornado.gen.coroutine
    def smembers(self, key):
        """sdiff

        Args:
            *keys (TYPE): Description

        Returns:
            TYPE: Description

        Raises:
            gen.Return: Description
        """
        _start = int(round(time.time() * 1000))
        client = yield self._get_connected_client()
        result = yield client.call("SMEMBERS", key)
        self.pool.release_client(client)
        _stop = int(round(time.time() * 1000)) - _start

        finbull.log.service_debug({
            "redis_log": "redis requested",
            "method": "SMEMBERS",
            "key": key,
            "result": result,
            "time": _stop,
        })
        finbull.log.service_notice({
            "redis_log": "redis requested",
            "method": "SMEMBERS",
            "key": key,
            "time": _stop,
        })
        raise tornado.gen.Return(result)

    @tornado.gen.coroutine
    def delete(self, keys):
        """delete

        Args:
            keys (TYPE): Description

        Returns:
            TYPE: Description

        Raises:
            gen.Return: Description
        """
        if not isinstance(keys, list):
            keys = [keys]

        _start = int(round(time.time() * 1000))
        client = yield self._get_connected_client()
        result = yield client.call("DEL", *keys)
        self.pool.release_client(client)
        _stop = int(round(time.time() * 1000)) - _start

        finbull.log.service_debug({
            "redis_log": "redis requested",
            "method": "DELETE",
            "key": key,
            "result": result,
            "time": _stop,
        })
        finbull.log.service_notice({
            "redis_log": "redis requested",
            "method": "DELETE",
            "key": key,
            "time": _stop,
        })
        raise tornado.gen.Return(result)



################################################################################
#
# the Huskar framework
# @author dongliqiang@baidu.com
# Copyright (c) 2016 Baidu.com, Inc. All Rights Reserved
#
################################################################################

import os
import io
import finbull.error
import validate
from configobj import ConfigObj

# parameter Validator


def _validator_is_dict(value):
    """
    extents for ConfigObj Vaildator
    """
    if not isinstance(value, dict):
        raise validate.VdtTypeError(value)
    return value

#global validator
VALIDATOR = validate.Validator()
VALIDATOR.functions["dict"] = _validator_is_dict


class Conf(object):
    """
    Huskar Conf Class
    adapter for baidu idc
    """

    _IDC_PREFIX = ".IDC_"
    _VALID_SUFFIX = ".valid"
    _ENCODEING = "UTF-8"
    _INDENT_TYPE = None
    _conf = None

    def __init__(self, **kwargs):
        """
        super __init__
        """
        infile = kwargs["infile"] or None
        self._idc = kwargs["_idc"] if "_idc" in kwargs else None
        if self._idc is not None:
            del kwargs["_idc"]

        kwargs["encoding"] = self._ENCODEING
        kwargs["default_encoding"] = self._ENCODEING
        kwargs["indent_type"] = self._INDENT_TYPE
        kwargs["write_empty_values"] = True

        if infile is not None:
            # you can ONLY read conf
            if isinstance(infile, str) and \
                    os.path.isfile(infile):
                # validation
                infile_valid = infile + self._VALID_SUFFIX
                if os.path.isfile(infile_valid):
                    kwargs["infile"] = infile
                    kwargs["configspec"] = infile_valid
                    self._conf = ConfigObj(**kwargs)
                    test = self._conf.validate(VALIDATOR)
                    if test is not True:
                        raise finbull.error.BaseError(
                            errno=finbull.error.ERRNO_FRAMEWORK,
                            errmsg="conf vaild failed.[%s]" % str(test))
                else:
                    # in this framework
                    # we MUST use the validation
                    raise finbull.error.BaseError(
                        errno=finbull.error.ERRNO_FRAMEWORK,
                        errmsg="can not find valid file for [%s]"
                               " needed valid file [%s]" % (infile, infile_valid))

            if isinstance(infile, io.StringIO) and \
                    getattr(infile, "read") is not None:
                self._conf = ConfigObj(**kwargs)

        if self._conf is None:
            raise finbull.error.BaseError(
                errno=finbull.error.ERRNO_FRAMEWORK,
                errmsg="conf is not found. [%s]" % infile)

        self._conf.walk(self._parse_idc, call_on_sections=True)

    def _parse_idc(self, section, key):
        """
        get the configuration by the current idc
        """
        value = section[key]
        if isinstance(value, dict) and len(value) > 0:
            keys = list(value.keys())
            cur_idc = self._IDC_PREFIX + self._idc
            idc_mode_keys = [x for x in keys if x.startswith(self._IDC_PREFIX)]
            if len(idc_mode_keys) > 0:
                # in idc mode
                matches_idc = [x for x in idc_mode_keys if x == cur_idc]
                if len(matches_idc) == 1:
                    # find current idc
                    section[key] = self._merge_dicts(section[key], value[cur_idc])
                    del section[key][cur_idc]
                else:
                    # can not find current idc in conf
                    raise finbull.error.BaseError(
                        errno=finbull.error.ERRNO_FRAMEWORK,
                        errmsg="useing idc mode, can not find matched idc[%s] in file[%s]"
                               % ((cur_idc), self._conf.filename))

            # delete all idc in every section
            for k in keys:
                if k.startswith(self._IDC_PREFIX) and k != cur_idc:
                    del section[key][k]

    def _merge_dicts(self, *dict_args):
        """
        Given any number of dicts, shallow copy and merge into a new dict,
        precedence goes to key value pairs in latter dicts.
        """
        result = {}
        for dictionary in dict_args:
            result.update(dictionary)
        return result

    def __getitem__(self, key):
        """
        hack for __getitem__
        adapter for baidu idc
        """
        return self._conf.__getitem__(key)

    def __setitem__(self, key, value):
        """
        hack for __setitem__
        you CAN NOT set value to the configuration
        """
        pass

    def __repr__(self):
        """
        __repr__
        """
        return repr(self._conf)

    def __str__(self):
        """
        __str__
        """
        return str(self._conf)


if __name__ == '__main__':

    input_config = u"""
    [s1]
    author = dongliqiang
    description = Test config

    [s2]
    [[.IDC_nj]]
    key1 = aaa
    [[.IDC_sh]]
    key1 = bbb

    [s3]
    key3 = abc
    [[s4]]
    [[[.IDC_nj]]]
    key2 = ccc
    [[[.IDC_sh]]]
    key2 = ddd
    """
    conf = Conf(infile=io.StringIO(input_config), _idc="nj")
    print(conf, conf["s2"])



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2019 hcyjs.com, Inc. All Rights Reserved

"""
File: gremlin_server.py.py
Author: wangjiangfeng(wangjiangfeng@pku.edu.cn)
Date: 2019-07-31 20:01
"""

import random
import requests
import json
import finbull.error
import finbull.log


class GremlinServer(object):
    """Summary

    Attributes
        ip_list (list or str): [IP:Port, IP:Port]
        username (str): username
        password (str): password
    """

    def __init__(self, ip_list=None, **kwargs):
        """
        初始化函数
        :param ip_list:
        :param kwargs:
        """
        self.ip_list = ip_list
        self.client_kwargs = kwargs

        if self.ip_list is None:
            raise finbull.error.BaseError(
                errno=finbull.error.ERRMSG_FRAMEWORK,
                errmsg="ip_list is empty both."
            )
        if not isinstance(self.ip_list, list):
            self.ip_list = [self.ip_list]

        self.username = kwargs["username"] if "username" in kwargs else ""
        self.password = kwargs["password"] if "password" in kwargs else ""

    def _get_client(self):
        """

        :return:
        """
        if self.ip_list is not None:
            server = random.choice(self.ip_list)
            (ip, port) = server.split(":")
            return "http://%s:%s" % (ip, port)
        else:
            raise finbull.error.BaseError(
                errno=finbull.error.ERRNO_FRAMEWORK,
                errmsg="Gremlin Server Ip List Error"
            )

    def query(self, cmd):
        """

        :param cmd:
        :return:
        """
        return self._send_request(cmd)

    def _send_request(self, cmd):
        """

        :param cmd:
        :return:
        """
        header = {"Content-Type": "application/json; charset=UTF-8", 'Connection': 'close'}
        request_data = dict()
        request_data["gremlin"] = cmd
        request_url = self._get_client()
        r = requests.post(request_url, data=json.dumps(request_data), headers=header)
        response_data = r.text
        finbull.log.service_debug({
            "request_url": request_url,
            "request_data": request_data
        })
        return json.loads(response_data)
